//---------------------------------------------------------------------
// <copyright file="Default.aspx.cs" company="Joris Kalz">
//      Copyright (c) Joris Kalz.  All rights reserved.
// </copyright>
// <summary>
//      Default.aspx.cs class file for the WDS sample. 
// </summary>
//---------------------------------------------------------------------

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;


using System.Collections;

// using the generic databinder sample from the SDK
using DataBinder;
using DataBinder.CrmSdk;
using DataBinder.MetadataServiceSdk;

public partial class _Default : System.Web.UI.Page 
{

    private UniversalEntityList _data;

    protected void Page_Load(object sender, EventArgs e)
    {
        // Clear all messages
        LabelError.Text = "";
        LabelResult.Text = "";

        // Reset the Datalist
        DataList2.DataBind();
        string searchString = "";

        // Check, if the searchstring is coming from the
        // querystring or from the text input field
        if (IsPostBack)
        {
            searchString = TextBoxQuery.Text;
        }
        else
        {
            if (Request.QueryString["q"] != null)
            {
                TextBoxQuery.Text = Request.QueryString["q"];
                searchString = Request.QueryString["q"];
            }
        }


        // searchstring must be at least two characters long
        if (searchString.Length >= 2)
        {

            // get the result for the searchstring
            DataSet dataSet = queryCRM(searchString);

            // do we have at leat one record found?
            if (dataSet.Tables[0].Rows.Count > 0)
            {
                try
                {

                    if (dataSet.Tables[0].Rows.Count > 500)
                        throw new ApplicationException("More than 500 records where found. Please refine your query.");

                    // binding the data to the datalist
                    this.DataList2.DataSource = dataSet;
                    this.DataList2.DataBind();

                    // just giving the user some information
                    // about the number of records found in the database
                    if (dataSet.Tables[0].Rows.Count == 1)
                        LabelResult.Text = string.Format("Found {0} result in Microsoft CRM", dataSet.Tables[0].Rows.Count.ToString());
                    else
                        LabelResult.Text = string.Format("Found {0} results in Microsoft CRM", dataSet.Tables[0].Rows.Count.ToString());

                }
                catch (Exception ex)
                {
                    LabelError.Text = "Error: " + ex.Message;
                    LabelError.Visible = true;
                }
            }
            else
            {
                // no record found in Microsoft CRM
                LabelError.Text = "Could not find any data.";
                LabelError.Visible = true;
            }
        }
        else
        {
            LabelError.Text = "Query must be at least 2 characters long";
            LabelError.Visible = true;
        }

  
  

     
    }

    /// <summary>
    /// Display all search results, accounts and contacts
    /// This handles the OnItemCreated event.
    /// </summary>
    /// <param name="sender">Calling object</param>
    /// <param name="e">Event arguments</param>
    protected void SetValues(object sender, DataListItemEventArgs e)
    {
        // Get the row and label associated with the current entity.
        DataRowView rowView = (DataRowView)e.Item.DataItem;
        System.Web.UI.WebControls.Image image = ((System.Web.UI.WebControls.Image)e.Item.FindControl("ImageType"));


        // this could be done without error handling but I had not the time...
        try
        {
            string entitytype = (string)rowView["type"];

            switch (entitytype)
            {
                case "account":
                    image.ImageUrl = "ico_16_1.gif";
                    break;
                case "contact":
                    image.ImageUrl = "ico_16_2.gif";
                    break;
            }

        }
        catch (Exception)
        {
            //throw;
        }
    }


    /// <summary>
    /// Returns all accounts from Microsoft CRM 
    /// for a given search string
    /// </summary>
    /// <param name="query">Searchstring</param>
    private BusinessEntityCollection getAccounts(string query)
    {
        // Set up the CRM Service.\
        CrmService service = new CrmService();
        service.Credentials = System.Net.CredentialCache.DefaultCredentials;

        // Create a column set holding the names of the columns 
        // to be retrieved.
        ColumnSet colsPrincipal = new ColumnSet();

        // Set the properties of the column set.
        // TODO: make it generic
        colsPrincipal.Attributes = new string[] { "accountid", "name" };

        // Create a ConditionExpression.
        ConditionExpression conditionPrincipal = new ConditionExpression();

        // Set the ConditionExpressions properties so that the condition is
        // true when the account name is like the search string.
        conditionPrincipal.AttributeName = "name";
        conditionPrincipal.Operator = ConditionOperator.Like;
        conditionPrincipal.Values = new object[1];
        conditionPrincipal.Values[0] = string.Format("%{0}%", query);

        // Create the FilterExpression.
        FilterExpression filterPrincipal = new FilterExpression();

        // Set the properties of the FilterExpression.
        filterPrincipal.FilterOperator = LogicalOperator.And;
        filterPrincipal.Conditions = new ConditionExpression[] { conditionPrincipal };

        // Create the QueryExpression.
        QueryExpression queryPrincipal = new QueryExpression();

        // Set the properties of the QueryExpression.
        queryPrincipal.EntityName = EntityName.account.ToString();
        queryPrincipal.ColumnSet = colsPrincipal;
        queryPrincipal.Criteria = filterPrincipal;

        // Create the request object.
        RetrieveMultipleRequest retrievePrincipal = new RetrieveMultipleRequest();

        // Set the properties of the request object.
        retrievePrincipal.Query = queryPrincipal;
        retrievePrincipal.ReturnDynamicEntities = true; // The UniversalEntityList only supports dynamic entities

        // Execute the request.
        RetrieveMultipleResponse principalResponse = (RetrieveMultipleResponse)service.Execute(retrievePrincipal);

        return principalResponse.BusinessEntityCollection;
    }


    /// <summary>
    /// Returns all contacts from Microsoft CRM 
    /// for a given search string
    /// </summary>
    /// <param name="query">Searchstring</param>
    private BusinessEntityCollection getContacts(string query)
    {
        // Set up the CRM Service.\
        CrmService service = new CrmService();
        service.Credentials = System.Net.CredentialCache.DefaultCredentials;

        // Create a column set holding the names of the columns 
        // to be retrieved.
        ColumnSet colsPrincipal = new ColumnSet();

        // Set the properties of the column set.
        colsPrincipal.Attributes = new string[] { "contactid", "lastname", "firstname" };

        // Create two ConditionExpression, one for lastname, one for firstname
        ConditionExpression conditionPrincipal1 = new ConditionExpression();
        ConditionExpression conditionPrincipal2 = new ConditionExpression();

        // Set the ConditionExpressions properties so that the condition is
        // true when the contact lastname is like the query.
        conditionPrincipal1.AttributeName = "lastname";
        conditionPrincipal1.Operator = ConditionOperator.Like;
        conditionPrincipal1.Values = new object[1];
        conditionPrincipal1.Values[0] = string.Format("%{0}%", query);

        // Set the ConditionExpressions properties so that the condition is
        // true when the contact firstname is like the query.
        conditionPrincipal2.AttributeName = "firstname";
        conditionPrincipal2.Operator = ConditionOperator.Like;
        conditionPrincipal2.Values = new object[1];
        conditionPrincipal2.Values[0] = string.Format("%{0}%", query);

        // Create the FilterExpression.
        FilterExpression filterPrincipal = new FilterExpression();

        // Set the properties of the FilterExpression.
        filterPrincipal.FilterOperator = LogicalOperator.Or;
        filterPrincipal.Conditions = new ConditionExpression[] { conditionPrincipal1, conditionPrincipal2 };

        // Create the QueryExpression.
        QueryExpression queryPrincipal = new QueryExpression();

        // Set the properties of the QueryExpression.
        queryPrincipal.EntityName = EntityName.contact.ToString();
        queryPrincipal.ColumnSet = colsPrincipal;
        queryPrincipal.Criteria = filterPrincipal;

        // Create the request object.
        RetrieveMultipleRequest retrievePrincipal = new RetrieveMultipleRequest();

        // Set the properties of the request object.
        retrievePrincipal.Query = queryPrincipal;
        retrievePrincipal.ReturnDynamicEntities = true; // The UniversalEntityList only supports dynamic entities

        // Execute the request.
        RetrieveMultipleResponse principalResponse = (RetrieveMultipleResponse)service.Execute(retrievePrincipal);

        return principalResponse.BusinessEntityCollection;
    }

    /// <summary>
    /// Returns all contacts and accounts from Microsoft CRM 
    /// for a given search string
    /// </summary>
    /// <param name="query">Searchstring</param>
    private DataSet queryCRM(string query)
    {

        // set up a generic table which holgs the result for accounts and contacts
        DataTable dataTable = new DataTable();
        dataTable.CaseSensitive = false;

        dataTable.Columns.Add("type");
        dataTable.Columns.Add("id");
        dataTable.Columns.Add("field1");
        dataTable.Columns.Add("field2");

        // get the accounts for the query
        BusinessEntityCollection entityCollectionAccounts = getAccounts(query);


        // adding accounts to the data table
        for (int i = 0; i < entityCollectionAccounts.BusinessEntities.Length; i++)
        {
            DynamicEntity dynamicEntity = (DynamicEntity)entityCollectionAccounts.BusinessEntities[i];


            DataRow dataRow = dataTable.NewRow();


            dataRow["type"] = dynamicEntity.Name;

            // the next two lines works but I don't know why... please explain it to me
            // when KeyProperty and when StringProperty???
            // there must be an easier why
            dataRow["id"] = ((KeyProperty)dynamicEntity.Properties[0]).Value.Value;
            dataRow["field1"] = ((StringProperty)dynamicEntity.Properties[1]).Value;
            // TODO: adding more information for e.g. city as a differentiator
            dataRow["field2"] = "";
            
            dataTable.Rows.Add(dataRow);
        }


        // second, get all the contacts from Microsoft CRM for the given query
        BusinessEntityCollection entityCollectionContacts = getContacts(query);


        // adding contacts two the data table
        for (int i = 0; i < entityCollectionContacts.BusinessEntities.Length; i++)
        {
            DynamicEntity dynamicEntity = (DynamicEntity)entityCollectionContacts.BusinessEntities[i];


            DataRow dataRow = dataTable.NewRow();

            dataRow["type"] = dynamicEntity.Name;

            // the next two lines works but I don't know why... please explain it to me
            // when KeyProperty and when StringProperty???
            // there must be an easier why
            dataRow["id"] = ((KeyProperty)dynamicEntity.Properties[0]).Value.Value;
            dataRow["field1"] = ((StringProperty)dynamicEntity.Properties[1]).Value;
            dataRow["field2"] = ((StringProperty)dynamicEntity.Properties[2]).Value;

            dataTable.Rows.Add(dataRow);
        }


        // returning the table inside a dataset so that we could bind it to
        // our Datalist.
        DataSet dataSet = new DataSet();
        dataSet.Tables.Add(dataTable);
        
        return dataSet;

    }

    protected void DataList2_ItemCreated(object sender, DataListItemEventArgs e)
    {

    }

    // Just some encoding for special characters
    protected string GetCellEntry(object o)
    {
        string text = o.ToString();
        if (text != null && text.Trim() != string.Empty)
            return Server.HtmlEncode(text);
        else
            return "&nbsp;";
    }

    // not used yet, but it should...
    // TODO: implement web.config settings
    protected string GetCRMServer()
    {
        return System.Web.Configuration.WebConfigurationManager.AppSettings["crmserver"]; 
    }

}
